@@ -17,10 +17,12 @@
 System-level utilities and helper functions.
 """
 
+import copy
 import math
 import re
 import unicodedata
 
+import pyparsing as pp
 import six
 from six.moves import urllib
 
@@ -55,7 +57,7 @@ SLUGIFY_HYPHENATE_RE = re.compile(r"[-\s]+")
 _SANITIZE_KEYS = ['adminPass', 'admin_pass', 'password', 'admin_password',
                   'auth_token', 'new_pass', 'auth_password', 'secret_uuid',
                   'secret', 'sys_pswd', 'token', 'configdrive',
-                  'CHAPPASSWORD']
+                  'CHAPPASSWORD', 'encrypted_key']
 
 # NOTE(ldbragst): Let's build a list of regex objects using the list of
 # _SANITIZE_KEYS we already have. This way, we only have to add the new key
@@ -109,11 +111,14 @@ def int_from_bool_as_string(subject):
 
 
 def bool_from_string(subject, strict=False, default=False):
-    """Interpret a string as a boolean.
+    """Interpret a subject as a boolean.
 
-    A case-insensitive match is performed such that strings matching 't',
-    'true', 'on', 'y', 'yes', or '1' are considered True and, when
-    `strict=False`, anything else returns the value specified by 'default'.
+    A subject can be a boolean, a string or an integer. Boolean type value
+    will be returned directly, otherwise the subject will be converted to
+    a string. A case-insensitive match is performed such that strings
+    matching 't','true', 'on', 'y', 'yes', or '1' are considered True and,
+    when `strict=False`, anything else returns the value specified by
+    'default'.
 
     Useful for JSON-decoded stuff and config file parsing.
 
@@ -143,6 +148,19 @@ def bool_from_string(subject, strict=False, default=False):
         return default
 
 
+def is_valid_boolstr(value):
+    """Check if the provided string is a valid bool string or not.
+
+    :param value: value to verify
+    :type value: string
+    :returns: true if value is boolean string, false otherwise
+
+    .. versionadded:: 3.17
+    """
+    boolstrs = TRUE_STRINGS + FALSE_STRINGS
+    return str(value).lower() in boolstrs
+
+
 def string_to_bytes(text, unit_system='IEC', return_int=False):
     """Converts a string into an float representation of bytes.
 
@@ -305,16 +323,16 @@ def mask_dict_password(dictionary, secret="***"):  # nosec
     A dictionary (which may contain nested dictionaries) contains
     information (such as passwords) which should not be revealed, and
     this function helps detect and replace those with the 'secret'
-    provided (or '***' if none is provided).
+    provided (or `***` if none is provided).
 
     Substitution is performed in one of three situations:
 
     If the key is something that is considered to be indicative of a
     secret, then the corresponding value is replaced with the secret
-    provided (or '***' if none is provided).
+    provided (or `***` if none is provided).
 
     If a value in the dictionary is a string, then it is masked
-    using the mask_password() function.
+    using the ``mask_password()`` function.
 
     Finally, if a value is a dictionary, this function will
     recursively mask that dictionary as well.
@@ -353,18 +371,23 @@ def mask_dict_password(dictionary, secret="***"):  # nosec
         raise TypeError("Expected a dictionary, got %s instead."
                         % type(dictionary))
 
-    out = {}
+    out = copy.deepcopy(dictionary)
 
     for k, v in dictionary.items():
         if isinstance(v, dict):
-            v = mask_dict_password(v, secret=secret)
-        elif k in _SANITIZE_KEYS:
-            v = secret
-        elif isinstance(v, six.string_types):
-            v = mask_password(v, secret=secret)
-
-        out[k] = v
-
+            out[k] = mask_dict_password(v, secret=secret)
+            continue
+        # NOTE(jlvillal): Check to see if anything in the dictionary 'key'
+        # contains any key specified in _SANITIZE_KEYS.
+        for sani_key in _SANITIZE_KEYS:
+            if sani_key in k:
+                out[k] = secret
+                break
+        else:
+            # We did not find a match for the key name in the
+            # _SANITIZE_KEYS, so we fall through to here
+            if isinstance(v, six.string_types):
+                out[k] = mask_password(v, secret=secret)
     return out
 
 
@@ -462,3 +485,20 @@ def split_path(path, minsegs=1, maxsegs=None, rest_with_last=False):
     segs = segs[1:maxsegs]
     segs.extend([None] * (maxsegs - 1 - len(segs)))
     return segs
+
+
+def split_by_commas(value):
+    """Split values by commas and quotes according to api-wg
+
+    :param value: value to be split
+
+    .. versionadded:: 3.17
+    """
+    word = (pp.QuotedString(quoteChar='"', escChar='\\')
+            | pp.Word(pp.printables, excludeChars='",'))
+    grammar = pp.stringStart + pp.delimitedList(word) + pp.stringEnd
+
+    try:
+        return list(grammar.parseString(value))
+    except pp.ParseException:
+        raise ValueError("Invalid value: %s" % value)
